import sys
import atproto_client
from ssky.profile_list import ProfileList
from ssky.ssky_session import expand_actor, ssky_client

def profile(name, **kwargs) -> ProfileList:
    try:
        profile = ssky_client().get_profile(expand_actor(name))
        if profile is None:
            print(f'Profile not found', file=sys.stderr)
            return None
        return ProfileList().append(profile.did)
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import sys
import atproto_client
from ssky.ssky_session import ssky_client
from ssky.util import disjoin_uri_cid, is_joined_uri_cid

def delete(post, **kwargs) -> str:
    if is_joined_uri_cid(post):
        uri, _ = disjoin_uri_cid(post)
    else:
        uri = post

    try:
        status = ssky_client().delete_post(uri)
        if status is False:
            print('Failed to delete', file=sys.stderr)
            return None
        return uri
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import re
import sys
from time import sleep
from atproto import IdResolver, models
import atproto_client
from bs4 import BeautifulSoup
import requests
from ssky.ssky_session import ssky_client
from ssky.post_data_list import PostDataList
from ssky.util import disjoin_uri_cid, is_joined_uri_cid

def get_card(links):
    title = None
    description = None

    headers = { 'Cache-Control': 'no-cache', 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.61 Safari/537.36' }

    for key in links:
        uri = links[key]['uri']

        res = None
        try:
            res = requests.get(uri, headers=headers)
        except Exception as e:
            error_message = str(e)
            print(f'{error_message}', file=sys.stderr)
            continue

        if res.status_code >= 400:
            if res.status_code == 403:
                headers_no_user_agent = { 'Cache-Control': 'no-cache' }

                res_no_user_agent = None
                try:
                    res_no_user_agent = requests.get(uri, headers=headers_no_user_agent)
                except Exception as e:
                    pass

                if res_no_user_agent is not None:
                    res = res_no_user_agent

            if res.status_code >= 400:
                error = ' '.join([str(res.status_code), res.text if res.text is not None else ''])
                print(f'{error} ', file=sys.stderr)
                continue

        if not 'Content-Type' in res.headers:
            print('No Content-Type', file=sys.stderr)
            continue

        content_type_fragments = res.headers['Content-Type'].split(';')

        mime_type = content_type_fragments[0].strip().lower()
        if mime_type != 'text/html':
            print(f'Unexpected mime type {mime_type}', file=sys.stderr)
            continue

        if len(content_type_fragments) < 2:
            print(f'Warning: get_card: No charset; assume utf-8', file=sys.stderr)
        else:
            charset = content_type_fragments[1].split('=')[1].strip().lower()
            if charset != 'utf-8':
                print(f'Unexpected charset {charset}', file=sys.stderr)
                continue

        if len(res.text) == 0:
            print('Empty content', file=sys.stderr)
            continue

        soup = BeautifulSoup(res.content, 'html.parser')

        title = 'No title'
        result = soup.find('title')
        if result is not None:
            title = result.text
        else:
            result = soup.find('meta', attrs={'property': 'og:title'})
            if result is not None:
                title = result.get('content')

        description = uri
        result = soup.find('meta', attrs={'name': 'description'})
        if result is not None:
            description = result.get('content')
        else:
            result = soup.find('meta', attrs={'property': 'og:description'})
            if result is not None:
                description = result.get('content')

        thumbnail = None
        result = soup.find('meta', attrs={'property': 'og:image'})
        if result is not None:
            thumbnail = result.get('content')
            if len(thumbnail) == 0:
                thumbnail = None

        return {
            'title': title,
            'description': description,
            'thumbnail': thumbnail,
            'uri': uri
        }

    return None

def byte_len(text):
    return len(text.encode('UTF-8'))

def search_items(text, pattern, property_name):
    matches = re.finditer(pattern, text)
    items = {}
    for m in matches:
        byte_start = byte_len(text[:m.start()])
        byte_end = byte_start + byte_len(m.group())
        items[f'{m.start():05d}'] = {
            'byte_start': byte_start,
            'byte_end': byte_end,
            'start': m.start(),
            'end': m.end(),
            property_name: m.group()
        }
    return items

def get_links(message):
    return search_items(message, r'https?://[\w/:%#\$&\?\(\)~\.=\+\-]+', 'uri')

def get_tags(message):
    return search_items(message, r'#\S+', 'name')

def get_mentions(message):
    mentions = search_items(message, r'@[\w.]+', 'handle')
    for key in mentions:
        name = mentions[key]['handle'][1:]
        resolver = IdResolver()
        did = resolver.handle.resolve(name)
        mentions[key]['did'] = did
    return mentions

def get_thumbnail(uri):
    headers = { 'Cache-Control': 'no-cache', 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.61 Safari/537.36' }

    res = None
    try:
        res = requests.get(uri, headers=headers)
    except Exception as e:
        error_message = str(e)
        print(f'{error_message}', file=sys.stderr)
        return None

    if res.status_code >= 400:
        if res.status_code == 403:
            headers_no_user_agent = { 'Cache-Control': 'no-cache' }

            res_no_user_agent = None
            try:
                res_no_user_agent = requests.get(uri, headers=headers_no_user_agent)
            except Exception as e:
                pass

            if res_no_user_agent is not None:
                res = res_no_user_agent

        if res.status_code >= 400:
            error = ' '.join([str(res.status_code), res.text if res.text is not None else ''])
            print(f'{error} ', file=sys.stderr)
            return None

    if not 'Content-Type' in res.headers:
        print('No Content-Type', file=sys.stderr)
        return None

    content_type_fragments = res.headers['Content-Type'].split(';')
    mime_type = content_type_fragments[0].strip().lower()
    if mime_type != 'image/jpeg' and mime_type != 'image/png' and mime_type != 'image/gif':
        print(f'Unexpected mime type {mime_type}', file=sys.stderr)
        return None

    return res.content

def load_images(paths):
    images = []
    for path in paths:
        with open(path, 'rb') as f:
            images.append(f.read())
    return images

def get_post(uri_cid):
    if is_joined_uri_cid(uri_cid):
        uri, cid = disjoin_uri_cid(uri_cid)
    else:
        uri = uri_cid
        cid = None
    retrieved_posts =ssky_client().get_posts([uri])
    posts = []
    for retrieved_post in retrieved_posts.posts:
        if retrieved_post.uri == uri and (cid is None or retrieved_post.cid == cid):
            posts.append(retrieved_post)

    if len(posts) == 0:
        return None

    return posts[0]

def get_root_strong_ref(post):
    slug = post.uri.split('/')[-1]
    did = post.author.did
    cid = post.cid
    retrieved_post = ssky_client().get_post(slug, profile_identify=did,cid=cid)
    if retrieved_post.value.reply is None:
        return models.create_strong_ref(post)
    else:
        return retrieved_post.value.reply.root

def post(message=None, dry=False, image=[], quote=None, reply_to=None, **kwargs) -> PostDataList:
    if message:
        message = message
    else:
        message = sys.stdin.read()

    message = message.strip()

    tags = get_tags(message)
    links = get_links(message)
    mentions = get_mentions(message)

    card = None
    if len(image) == 0 and not quote:
        card = get_card(links)

    if dry:
        result = []
        result.append([message])
        for key in tags:
            result.append(['Tag', tags[key]['name']])
        for key in links:
            result.append(['Link', links[key]['uri']])
        for key in mentions:
            result.append(['Mention', mentions[key]['did'], mentions[key]['handle']])
        for img in image:
            result.append(['Image', img])
        if card is not None:
            result.append(['Card', card["uri"], card['title'], card['description'], card['thumbnail']])
        if reply_to:
            result.append(['Reply to', reply_to])
        return result

    facets = []
    for key in tags:
        facets.append(
            models.AppBskyRichtextFacet.Main(
                features=[models.AppBskyRichtextFacet.Tag(tag=tags[key]['name'][1:])],
                index=models.AppBskyRichtextFacet.ByteSlice(byte_start=tags[key]['byte_start'], byte_end=tags[key]['byte_end'])
            )
        )
    for key in links:
        facets.append(
            models.AppBskyRichtextFacet.Main(
                features=[models.AppBskyRichtextFacet.Link(uri=links[key]['uri'])],
                index=models.AppBskyRichtextFacet.ByteSlice(byte_start=links[key]['byte_start'], byte_end=links[key]['byte_end'])
            )
        )

    for key in mentions:
        facets.append(
            models.AppBskyRichtextFacet.Main(
                features=[models.AppBskyRichtextFacet.Mention(did=mentions[key]['did'])],
                index=models.AppBskyRichtextFacet.ByteSlice(byte_start=mentions[key]['byte_start'], byte_end=mentions[key]['byte_end'])
            )
        )

    try:
        reply_ref = None
        if reply_to:
            post_to_reply_to = get_post(reply_to)
            if post_to_reply_to is None:
                print('Reply target is missing', file=sys.stderr)
                return None
            reply_ref = models.app.bsky.feed.post.ReplyRef(
                parent=models.create_strong_ref(post_to_reply_to),
                root=get_root_strong_ref(post_to_reply_to)
            )

        if card is not None:
            thumb_blob_ref = None
            if card['thumbnail'] is not None:
                image = get_thumbnail(card['thumbnail'])
                if image is not None:
                    res = ssky_client().upload_blob(image)
                    if res.blob is None:
                        print('Failed to upload thumbnail', file=sys.stderr)
                        return None
                    thumb_blob_ref = res.blob

            embed_external = models.AppBskyEmbedExternal.Main(
                external = models.AppBskyEmbedExternal.External(
                    title = card['title'],
                    description = card['description'],
                    uri = card['uri'],
                    thumb = thumb_blob_ref
                )
            )
            res = ssky_client().send_post(text=message, facets=facets, embed=embed_external, reply_to=reply_ref)
        elif quote is not None:
            source = get_post(quote)
            if source is None:
                print('Quote source is missing', file=sys.stderr)
                return None
            embed_record = models.AppBskyEmbedRecord.Main(
                record = models.ComAtprotoRepoStrongRef.Main(
                    uri = source.uri,
                    cid = source.cid
                )
            )
            res = ssky_client().send_post(text=message, facets=facets, embed=embed_record, reply_to=reply_ref)
        elif image is not None:
            if len(image) > 4:
                print('Too many image files', file=sys.stderr)
                return None
            images = load_images(image)
            res = ssky_client().send_images(text=message, facets=facets, images=images, reply_to=reply_ref)
        else:
            res = ssky_client().send_post(text=message, facets=facets, reply_to=reply_ref)

        post = get_post(res.uri)
        while post is None:
            print('waiting', file=sys.stderr)
            sleep(1)
            post = get_post(res.uri)

        return PostDataList().append(post)
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import datetime
import re
import sys
from atproto import models
import atproto_client
from ssky.ssky_session import expand_actor, ssky_client
from ssky.post_data_list import PostDataList

def expand_datetime(dt: str) -> str:
    if dt:
        if dt == 'today':
            ymd = datetime.datetime.now().strftime('%Y%m%d')
            return f'{ymd[:4]}-{ymd[4:6]}-{ymd[6:8]}T00:00:00Z'
        elif dt == 'yesterday':
            ymd = (datetime.datetime.now() - datetime.timedelta(days=1)).strftime('%Y%m%d')
            return f'{ymd[:4]}-{ymd[4:6]}-{ymd[6:8]}T00:00:00Z'
        elif re.match(r'^\d{14}$', dt):
            return f'{dt[:4]}-{dt[4:6]}-{dt[6:8]}T{dt[8:10]}:{dt[10:12]}:{dt[10:12]}Z'
        elif re.match(r'^\d{8}$', dt):
            return f'{dt[:4]}-{dt[4:6]}-{dt[6:8]}T00:00:00Z'
        else:
            return dt
    else:
        return None

def search(q='*', author=None, since=None, until=None, limit=100, **kwargs) -> PostDataList:
    since = expand_datetime(since)
    until = expand_datetime(until)

    try:
        res = ssky_client().app.bsky.feed.search_posts(
            models.AppBskyFeedSearchPosts.Params(
                author=expand_actor(author),
                limit=limit,
                q=q,
                since=since,
                until=until
            )
        )

        post_data_list = PostDataList()
        if res.posts and len(res.posts) > 0:
            for post in res.posts:
                post_data_list.append(post)
        return post_data_list
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import sys
import atproto_client
from ssky.profile_list import ProfileList
from ssky.ssky_session import SskySession

def login(credentials=None, **kwargs) -> ProfileList:
    handle = None
    password = None
    
    # Parse credentials if provided
    if credentials is not None and ':' in credentials:
        handle, password = credentials.split(':', 1)
    
    try:
        session = SskySession(handle=handle, password=password)
        session.persist()
        return ProfileList().append(session.profile().did)
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import sys
import atproto_client
from ssky.post_data_list import PostDataList
from ssky.ssky_session import ssky_client
from ssky.util import disjoin_uri_cid, is_joined_uri_cid

def unrepost(post, **kwargs) -> str:
    if is_joined_uri_cid(post):
        source_uri, source_cid = disjoin_uri_cid(post)
    else:
        source_uri = post
        source_cid = None

    try:
        post_data_list = PostDataList()
        sources = ssky_client().get_posts([source_uri])
        repost_uri = None
        for source_post in sources.posts:
            if source_post.uri == source_uri and (source_cid is None or source_post.cid == source_cid):
                post_data_list.append(source_post)
                repost_uri = source_post.viewer.repost
                break

        if repost_uri is None:
            print(f'Post not found', file=sys.stderr)
            return None

        status = ssky_client().unrepost(repost_uri)
        if status is False:
            print('Failed to unrepost', file=sys.stderr)
            return None

        return post_data_list
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import re

def summarize(source, length_max=0):
    if source is None:
        return ''
    else:
        summary = re.sub(r'\s', '_', ''.join(list(map(lambda c: c if c > ' ' else ' ', source.rstrip()))))
        if length_max > 0 and len(summary) > length_max:
            summary = ''.join(summary[:length_max - 2]) + '..'
        return summary

def join_uri_cid(uri, cid) -> str:
    return '::'.join([uri, cid])

def disjoin_uri_cid(uri_cid) -> tuple:
    pair = uri_cid.split('::', 1)
    return pair[0], pair[1]

def is_joined_uri_cid(uri_cid) -> bool:
    return '::' in uri_cid


import sys
import atproto_client
from ssky.profile_list import ProfileList
from ssky.ssky_session import expand_actor, ssky_client

def follow(name, **kwargs) -> str:
    try:
        client = ssky_client()
        actor = expand_actor(name)
        profile = client.get_profile(actor)
        client.follow(profile.did)
        return ProfileList().append(profile.did)
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import sys
import atproto_client
from ssky.ssky_session import expand_actor, ssky_client
from ssky.post_data_list import PostDataList
from ssky.util import disjoin_uri_cid, is_joined_uri_cid

def get_posts(client, uri, cid) -> None:
    res = client.get_posts([uri])
    post_data_list = PostDataList()
    for post in res.posts:
        if post.uri == uri and (cid is None or post.cid == cid):
            post_data_list.append(post)
    return post_data_list

def get_author_feed(client, user, limit=100) -> None:
    res = client.get_author_feed(user, limit=limit)
    post_data_list = PostDataList()
    for feed_post in res.feed:
        post_data_list.append(feed_post.post)
    return post_data_list

def get_timeline(client, limit=100) -> None:
    res = client.get_timeline(limit=limit)
    post_data_list = PostDataList()
    for feed_post in res.feed:
        post_data_list.append(feed_post.post)
    return post_data_list

def get(target=None, limit=100, **kwargs) -> PostDataList:
    try:
        client = ssky_client()
        if target is None:
            post_data_list = get_timeline(client, limit=limit)
        elif target.startswith('at://'):
            if is_joined_uri_cid(target):
                uri, cid = disjoin_uri_cid(target)
            else:
                uri = target
                cid = None
            post_data_list = get_posts(client, uri, cid)
        elif target.startswith('did:'):
            post_data_list = get_author_feed(client, target, limit=limit)
        else:
            actor = expand_actor(target)
            post_data_list = get_author_feed(client, actor, limit=limit)
        return post_data_list
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


import json
import os
import sys
import atproto
import atproto_client

class SskySession:

    class Session:
        def __init__(self, client=None, profile=None):
            self.client = client
            self.profile = profile

    config_path = os.path.expanduser('~/.ssky')
    session = None

    login_failed = Session()

    class Status:
        NOT_LOGGED_IN = 0,
        LOGGED_IN = 1,
        LOGIN_FAILED = 2

    @classmethod
    def at_login_internal(cls, handle=None, password=None, session_string=None) -> Session:
        client = atproto.Client()
        profile = client.login(login=handle, password=password, session_string=session_string)
        return cls.Session(client, profile)

    @classmethod
    def login_internal(cls, handle=None, password=None) -> None:
        if SskySession.session is None:
            var_user = os.environ.get('SSKY_USER')
            
            # Try session file first (most efficient)
            session_login_succeeded = False
            if os.path.exists(cls.config_path) and os.path.isfile(cls.config_path):
                with open(cls.config_path, 'r') as f:
                    persistent_config = json.load(f)
                    session_string = persistent_config.get('session_string')
                try:
                    cls.session = cls.at_login_internal(session_string=session_string)
                    session_login_succeeded = True
                except atproto_client.exceptions.AtProtocolError:
                    pass  # Will try fallback credentials
            
            # If session login failed or no session file exists, try other credentials
            if not session_login_succeeded:
                # Try command line arguments (explicit specification)
                if handle is not None and password is not None:
                    cls.session = cls.at_login_internal(handle=handle, password=password)
                # Try environment variable (fallback)
                elif var_user is not None:
                    handle, password = var_user.split(':', 1)
                    cls.session = cls.at_login_internal(handle=handle, password=password)
                else:
                    cls.session = cls.login_failed
                    raise atproto_client.exceptions.LoginRequiredError('No credentials found. Please set SSKY_USER or run ssky login handle:password')

    @classmethod
    def persist_internal(cls) -> None:
        if cls.session is not None and cls.session is not cls.login_failed:
            session_string = cls.session.client.export_session_string()
            with open(cls.config_path, 'w') as f:
                json.dump({
                    'session_string': session_string
                }, f)

    @classmethod
    def status(cls) -> int:
        if cls.session is None:
            return cls.Status.NOT_LOGGED_IN
        elif cls.session is cls.login_failed:
            return cls.Status.LOGIN_FAILED
        else:
            return cls.Status.LOGGED_IN

    @classmethod
    def clear(cls) -> None:
        cls.session = None

    def __init__(self, handle=None, password=None):
        SskySession.login_internal(handle=handle, password=password)

    def persist(self) -> None:
        if SskySession.status() == SskySession.Status.NOT_LOGGED_IN:
            raise atproto_client.exceptions.LoginRequiredError('Login first')
        elif SskySession.status() == SskySession.Status.LOGIN_FAILED:
            pass
        else:
            SskySession.persist_internal()

    def client(self) -> atproto.Client:
        if SskySession.status() == SskySession.Status.NOT_LOGGED_IN:
            raise atproto_client.exceptions.LoginRequiredError('Login first')
        elif SskySession.status() == SskySession.Status.LOGIN_FAILED:
            pass
        else:
            return SskySession.session.client

    def profile(self) -> atproto_client.models.AppBskyActorDefs.ProfileViewDetailed:
        if SskySession.status() == SskySession.Status.NOT_LOGGED_IN:
            raise atproto_client.exceptions.LoginRequiredError('Login first')
        elif SskySession.status() == SskySession.Status.LOGIN_FAILED:
            pass
        else:
            return SskySession.session.profile

def ssky_client(login_handle=None, login_password=None) -> atproto.Client:
    return SskySession(handle=login_handle, password=login_password).client()

def ssky_profile(login_handle=None, login_password=None) -> atproto_client.models.AppBskyActorDefs.ProfileViewDetailed:
    return SskySession(handle=login_handle, password=login_password).profile()

def expand_actor(name: str, login_handle=None, login_password=None) -> str:
    profile = SskySession(handle=login_handle, password=login_password).profile()
    if profile is None:
        return None
    else:
        if name == 'myself':
            actor = profile.did
        else:
            actor = name
        return actor


import sys
from atproto import models
import atproto_client
from ssky.profile_list import ProfileList
from ssky.ssky_session import ssky_client

def user(q, limit=100, **kwargs) -> ProfileList:
    try:
        res = ssky_client().app.bsky.actor.search_actors(
            models.AppBskyActorSearchActors.Params(
                limit=limit,
                q=q
            )
        )

        actor_list = ProfileList()
        for actor in res.actors:
            actor_list.append(actor.did)
        return actor_list
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None


from datetime import datetime
import os
import re
from atproto_client import models
from ssky.util import join_uri_cid, summarize

class PostDataList:

    class Item:
        post: models.AppBskyFeedDefs.PostView = None

        def __init__(self, post: models.AppBskyFeedDefs.PostView, profile: models.AppBskyActorDefs.ProfileViewDetailed = None, uri_cid: str = None) -> None:
            self.post = post
            if profile:
                self.post.author = models.AppBskyActorDefs.ProfileViewBasic(
                    associated=profile.associated,
                    avatar=profile.avatar,
                    created_at=profile.created_at,
                    did=profile.did,
                    display_name=profile.display_name,
                    handle=profile.handle,
                    labels=profile.labels,
                    viewer=profile.viewer
                )
            if uri_cid:
                self.post.uri, self.post.cid = uri_cid.split('::')

        def id(self) -> str:
            return join_uri_cid(self.post.uri, self.post.cid)

        def text_only(self) -> str:
            return self.post.record.text if self.post.record.text else ''

        def short(self, delimiter: str = None) -> str:
            if delimiter is None:
                delimiter = PostDataList.get_default_delimiter()
            uri_cid = self.id()
            author_did = self.post.author.did
            author_handle = self.post.author.handle
            display_name_summary = summarize(self.post.author.display_name)
            text_summary = summarize(self.post.record.text, length_max=40)
            return delimiter.join([uri_cid, author_did, author_handle, display_name_summary, text_summary])

        def long(self) -> str:
            text_summary = self.post.record.text if self.post.record.text else ''
            return '\n'.join(
                filter(
                    lambda x: x is not None,
                    [
                        f'Author-DID: {self.post.author.did}',
                        f'Author-Display-Name: {self.post.author.display_name}',
                        f'Author-Handle: {self.post.author.handle}',
                        f'Created-At: {self.post.record.created_at}',
                        f'Record-CID: {self.post.cid}',
                        f'Record-URI: {self.post.uri}',
                        f'Repost-URI: {self.post.viewer.repost}' if self.post.viewer and self.post.viewer.repost else None,
                        f'',
                        text_summary
                    ]
                )
            )

        def json(self) -> str:
            return models.utils.get_model_as_json(self.post)

        def simple_json(self) -> str:
            import json
            simplified = {
                "uri": self.post.uri,
                "cid": self.post.cid,
                "author": {
                    "did": self.post.author.did,
                    "handle": self.post.author.handle,
                    "display_name": self.post.author.display_name,
                    "avatar": self.post.author.avatar
                },
                "text": self.post.record.text if self.post.record.text else "",
                "created_at": self.post.record.created_at,
                "reply_count": self.post.reply_count if hasattr(self.post, 'reply_count') else 0,
                "repost_count": self.post.repost_count if hasattr(self.post, 'repost_count') else 0,
                "like_count": self.post.like_count if hasattr(self.post, 'like_count') else 0,
                "indexed_at": self.post.indexed_at if hasattr(self.post, 'indexed_at') else None
            }
            return json.dumps(simplified, ensure_ascii=False, separators=(',', ':'))

        def printable(self, format: str, delimiter: str = None) -> str:
            if format == 'id':
                return self.id()
            elif format == 'long':
                return self.long()
            elif format == 'text':
                return self.text_only()
            elif format == 'json':
                return self.json()
            elif format == 'simple_json':
                return self.simple_json()
            else:
                return self.short(delimiter=delimiter)

        def get_filename(self) -> str:
            iso_datetime_str = self.post.record.created_at
            if iso_datetime_str is None:
                iso_datetime_str = "0000-00-00T00:00:00.000Z"
            datetime_components = re.split(r'T|Z|-|:|\+|\.', iso_datetime_str) 
            formatted_datetime_str = ''.join(datetime_components[:6])
            filename = f"{self.post.author.handle}.{formatted_datetime_str}.txt"
            return filename

    default_delimiter = ' '

    @classmethod
    def set_default_delimiter(cls, delimiter: str) -> None:
        cls.default_delimiter = delimiter

    @classmethod
    def get_default_delimiter(cls) -> str:
        return cls.default_delimiter

    def __init__(self, default_delimiter: str = None) -> None:
        self.items = []
        if default_delimiter is not None:
            self.default_delimiter = default_delimiter

    def __str__(self) -> str:
        return str(self.items)

    def __len__(self) -> int:
        return len(self.items)

    def __iter__(self) -> 'PostDataList':
        self.index = 0
        return self

    def __next__(self) -> models.AppBskyFeedDefs.PostView:
        if self.index >= len(self.items):
            raise StopIteration
        item = self.items[self.index]
        self.index += 1
        return item.post

    def __getitem__(self, index: int) -> models.AppBskyFeedDefs.PostView:
        return self.items[index].post

    def append(self, post: models.AppBskyFeedDefs.PostView, profile: models.AppBskyActorDefs.ProfileViewDetailed = None, uri_cid: str = None) -> 'PostDataList':
        item = self.Item(post, profile=profile, uri_cid=uri_cid)
        if item.id() not in [i.id() for i in self.items]:
            self.items.append(item)
        return self

    def print(self, format: str, output: str = None, delimiter: str = None) -> None:
        if output:
            for item in self.items:
                filename = item.get_filename()
                path = os.path.join(output, filename)
                with open(path, 'w') as f:
                    f.write(item.printable(format, delimiter=delimiter))
                    f.write('\n')
        else:
            continued = False
            for item in self.items:
                if format == 'long':
                    if continued:
                        print('----------------')
                    else:
                        continued = True
                print(item.printable(format, delimiter=delimiter))


import sys
import atproto_client
from ssky.profile_list import ProfileList
from ssky.ssky_session import expand_actor, ssky_client, ssky_profile

def unfollow(name, **kwargs) -> ProfileList:
    try:
        res = ssky_client().get_follows(ssky_profile().did)
        actor = expand_actor(name)
        for follow in res.follows:
            if follow.did == actor or follow.handle == actor:
                if follow.viewer.following:
                    status = ssky_client().unfollow(follow.viewer.following)
                    if status is False:
                        print('Failed to unfollow', file=sys.stderr)
                        return None
                    else:
                        return ProfileList().append(follow.did)
        print(f'You are not following {actor}', file=sys.stderr)
        return None
    except atproto_client.exceptions.AtProtocolError as e:
        if 'response' in dir(e) and e.response is not None:
            print(f'{e.response.status_code} {e.response.content.message}', file=sys.stderr)
        elif str(e) is not None and len(str(e)) > 0:
            print(f'{str(e)}', file=sys.stderr)
        else:
            print(f'{e.__class__.__name__}', file=sys.stderr)
        return None